package corestr

import (
	"fmt"
	"strings"

	"gitlab.com/evatix-go/core/constants"
	"gitlab.com/evatix-go/core/coredata/corejson"
	"gitlab.com/evatix-go/core/internal/reflectinternal"
)

type KeyAnyValuePair struct {
	Key         string
	valueString SimpleStringOnce
	Value       interface{}
}

func (it *KeyAnyValuePair) IsValueNull() bool {
	return it == nil || reflectinternal.IsNull(it.Value)
}

func (it *KeyAnyValuePair) HasNonNull() bool {
	return it != nil && reflectinternal.IsNotNull(it.Value)
}

func (it *KeyAnyValuePair) HasValue() bool {
	return it != nil && reflectinternal.IsNotNull(it.Value)
}

func (it *KeyAnyValuePair) IsValueEmptyString() bool {
	return it == nil || it.ValueString() == ""
}

func (it *KeyAnyValuePair) IsValueWhitespace() bool {
	return it == nil || strings.TrimSpace(it.ValueString()) == ""
}

func (it *KeyAnyValuePair) ValueString() string {
	if it.valueString.IsInitialized() {
		return it.valueString.String()
	}

	if it.HasNonNull() {
		valueString := fmt.Sprintf(constants.SprintValueFormat, it.Value)

		return it.
			valueString.
			GetPlusSetOnUninitialized(valueString)
	}

	return it.
		valueString.
		GetPlusSetEmptyOnUninitialized()
}

func (it *KeyAnyValuePair) Serialize() ([]byte, error) {
	return it.Json().Raw()
}

func (it *KeyAnyValuePair) ParseInjectUsingJson(
	jsonResult *corejson.Result,
) (*KeyAnyValuePair, error) {
	err := jsonResult.Unmarshal(it)

	if err != nil {
		return nil, err
	}

	return it, nil
}

// ParseInjectUsingJsonMust Panic if error
func (it *KeyAnyValuePair) ParseInjectUsingJsonMust(
	jsonResult *corejson.Result,
) *KeyAnyValuePair {
	parsed, err := it.
		ParseInjectUsingJson(jsonResult)

	if err != nil {
		panic(err)
	}

	return parsed
}

func (it KeyAnyValuePair) Json() corejson.Result {
	return corejson.New(it)
}

func (it *KeyAnyValuePair) JsonPtr() *corejson.Result {
	return corejson.NewPtr(it)
}

func (it *KeyAnyValuePair) AsJsonContractsBinder() corejson.JsonContractsBinder {
	return it
}

func (it *KeyAnyValuePair) AsJsoner() corejson.Jsoner {
	return it
}

func (it *KeyAnyValuePair) JsonParseSelfInject(
	jsonResult *corejson.Result,
) error {
	_, err := it.ParseInjectUsingJson(
		jsonResult,
	)

	return err
}

func (it *KeyAnyValuePair) AsJsonParseSelfInjector() corejson.JsonParseSelfInjector {
	return it
}

func (it KeyAnyValuePair) String() string {
	return fmt.Sprintf(
		keyValuePrintFormat,
		it.Key,
		it.Value)
}
